@@ -38,6 +38,10 @@
 #include <string.h>
 #include <gtk/gtk.h>
 
+#ifdef OS_OSX
+#include "osx/gedit-osx.h"
+#endif
+
 gboolean    
 gedit_help_display (GtkWindow   *parent,
 		    const gchar *name, /* "gedit" if NULL */
@@ -49,6 +53,17 @@ gedit_help_display (GtkWindow   *parent,
 	
 	g_return_val_if_fail ((parent == NULL) || GTK_IS_WINDOW (parent), FALSE);
 
+#ifdef OS_OSX
+	if (name == NULL || strcmp(name, "gedit.xml") == NULL || strcmp(name, "gedit") == 0)
+	{
+		return gedit_osx_show_help (link_id);
+	}
+	else
+	{
+		return FALSE;
+	}
+#endif
+
 	if (name == NULL)
 		name = "gedit";
 	else if (strcmp (name, "gedit.xml") == 0)
