@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: gedit-spell-plugin.c 6779 2009-01-01 15:41:34Z icq $
+ * $Id$
  */
 
 #ifdef HAVE_CONFIG_H
@@ -732,7 +732,6 @@ spell_cb (GtkAction   *action,
 	gint start, end;
 	GtkTextIter sel_start, sel_end;
 	gchar *word;
-	gboolean sel = FALSE;
 	gchar *data_dir;
 
 	gedit_debug (DEBUG_PLUGINS);
@@ -771,7 +770,6 @@ spell_cb (GtkAction   *action,
 		start = gtk_text_iter_get_offset (&sel_start);
 		end = gtk_text_iter_get_offset (&sel_end);
 		set_check_range (doc, start, end);
-		sel = TRUE;
 	}
 	else
 	{
